package twisk.modele;

import javafx.util.Pair;
import twisk.exceptions.ArcException;
import twisk.exceptions.ParametresException;
import twisk.outils.Observable;
import twisk.outils.Observateur;

import javafx.geometry.Point2D;
import java.util.*;

public class MondeIG implements Iterable<EtapeIG>, Observable {
    private final Map<String, EtapeIG> etapes;
    private int compteurActivites;
    private ArrayList<Observateur> observateurs;
    private ArrayList<ArcIG> arcs;
    private PointDeControleIG pointSelectionne;
    private List<Point2D> pointsCliques;
    private static final int NB_POINTS_COURBE = 4;
    private Set<String> etapesSelectionnees;
    private Set<ArcIG> arcsSelectionnes;
    private Set<String> entrees;
    private Set<String> sorties;



    public MondeIG() {
        etapes = new HashMap<>();
        arcs = new ArrayList<>();
        pointSelectionne = null;
        pointsCliques = new ArrayList<>();
        compteurActivites = 1;
        observateurs = new ArrayList<>();
        etapesSelectionnees = new HashSet<>();
        arcsSelectionnes = new HashSet<>();
        entrees = new HashSet<>();
        sorties = new HashSet<>();

        // Ajouter une activité par défaut à la construction
        ajouter("Activite");
    }


    /**
     * Ajoute une étape du type spécifié
     * @param type le type d'étape à ajouter
     */
    public void ajouter(String type) {
        if ("Activite".equals(type)) {
            String nom = "Activite " + compteurActivites++;
            ActiviteIG activite = new ActiviteIG(nom);
            etapes.put(activite.getIdentifiant(), activite);

            // Trace temporaire pour vérifier l'effet du clic
            System.out.println("Ajout d'une activité : " + nom + " (id: " + activite.getIdentifiant() + ")");

            // Notifier les observateurs
            notifierObservateurs();
        }
        // Autres types d'étapes pourront être ajoutés ici dans le futur
    }

    /**
     * Ajoute un arc entre deux points de contrôle
     * @param pt1 premier point de contrôle
     * @param pt2 deuxième point de contrôle
     * @throws ArcException si l'arc ne peut pas être ajouté
     */
    public void ajouter(PointDeControleIG pt1, PointDeControleIG pt2) throws ArcException {
        // Contrainte 4: Vérifier que les points sont différents
        if (pt1 == pt2) {
            throw new ArcException("Impossible de créer un arc: les deux points sont identiques");
        }

        // Contrainte 1: Vérifier que les points ne sont pas sur la même étape
        if (pt1.getEtape().equals(pt2.getEtape())) {
            throw new ArcException("Impossible de créer un arc: les deux points appartiennent à la même étape");
        }

        // Contrainte 2: Vérifier qu'il n'existe pas déjà un arc entre ces deux points
        for (ArcIG arc : arcs) {
            if ((arc.getP1().equals(pt1) && arc.getP2().equals(pt2))) {
                throw new ArcException("Impossible de créer un arc: un arc existe déjà entre ces deux points");
            }
        }

        // Contrainte 3: Vérifier qu'il n'existe pas déjà un arc dans le sens inverse
        for (ArcIG arc : arcs) {
            if ((arc.getP1().equals(pt2) && arc.getP2().equals(pt1)) ||
                    (arc.getP1().getEtape().equals(pt2.getEtape()) && arc.getP2().getEtape().equals(pt1.getEtape()))) {
                throw new ArcException("Impossible de créer un arc: un arc existe déjà dans le sens inverse");
            }
        }

        // Contrainte 5: Vérifier que le point de départ n'est pas déjà utilisé comme source d'un autre arc
        for (ArcIG arc : arcs) {
            if (arc.getP1().equals(pt1)) {
                throw new ArcException("Impossible de créer un arc: le point de départ est déjà utilisé comme source d'un autre arc");
            }
        }

        // Contrainte 6: Vérifier que le point d'arrivée n'est pas déjà utilisé comme destination d'un autre arc
        for (ArcIG arc : arcs) {
            if (arc.getP2().equals(pt2)) {
                throw new ArcException("Impossible de créer un arc: le point d'arrivée est déjà utilisé comme destination d'un autre arc");
            }
        }
        // Contrainte 7: Vérifier que l'ajout de cet arc ne crée pas de cycle
        if (creeraitUnCycle(pt1.getEtape(), pt2.getEtape())) {
            throw new ArcException("Impossible de créer un arc: cela créerait un cycle dans le graphe");
        }

        // Toutes les contraintes sont satisfaites, on peut ajouter l'arc
        ArcIG arc = new LigneDroiteIG(pt1, pt2);
        arcs.add(arc);
        notifierObservateurs();
    }

    /**
     * Gère la sélection d'un point de contrôle
     * @param pt le point de contrôle sélectionné
     * @throws ArcException si l'arc ne peut pas être créé
     */
    public void selectionnerPointDeControle(PointDeControleIG pt) throws ArcException {
        if (pointSelectionne == null) {
            // Premier point sélectionné
            pointSelectionne = pt;
        } else {
            // Deuxième point sélectionné, on tente de créer un arc
            PointDeControleIG premier = pointSelectionne;
            // Réinitialiser la sélection avant de tenter d'ajouter l'arc
            pointSelectionne = null;
            // Tenter d'ajouter l'arc (peut lever une ArcException)
            ajouter(premier, pt);
        }
    }



    /**
     * Retourne l'étape correspondant à l'identifiant
     * @param identifiant l'identifiant de l'étape
     * @return l'étape correspondante
     */
    public EtapeIG getEtape(String identifiant) {
        return etapes.get(identifiant);
    }

    /**
     * Retourne un itérateur sur les étapes
     * @return itérateur sur les étapes
     */
    @Override
    public Iterator<EtapeIG> iterator() {
        return etapes.values().iterator();
    }

    /**
     * Retourne un itérateur sur les arcs
     * @return itérateur sur les arcs
     */
    public Iterator<ArcIG> iteratorArcs() {
        return arcs.iterator();
    }

    @Override
    public void ajouterObservateur(Observateur o) {
        observateurs.add(o);
    }

    @Override
    public void notifierObservateurs() {
        for (Observateur o : observateurs) {
            o.reagir();
        }
    }
    /**
     * Gère un clic sur le monde
     * @param x coordonnée x du clic
     * @param y coordonnée y du clic
     * @throws ArcException si l'arc ne peut pas être créé
     */
    public void clic(double x, double y) throws ArcException {
        // Vérifier si le clic est sur un point de contrôle
        PointDeControleIG pointClique = null;

        // Parcourir toutes les étapes et leurs points de contrôle
        for (EtapeIG etape : etapes.values()) {
            for (PointDeControleIG pt : etape) {
                // Vérifier si le clic est sur ce point (distance < 10 pixels)
                double distance = Math.sqrt(Math.pow(pt.getPosX() - x, 2) + Math.pow(pt.getPosY() - y, 2));
                if (distance < 10) {
                    pointClique = pt;
                    break;
                }
            }
            if (pointClique != null) {
                break;
            }
        }

        // Si le clic est sur un point de contrôle
        if (pointClique != null) {
            // Ajouter le point à la liste des points cliqués
            pointsCliques.add(new Point2D(pointClique.getPosX(), pointClique.getPosY()));

            // Si c'est le premier point, le mémoriser comme point sélectionné
            if (pointsCliques.size() == 1) {
                pointSelectionne = pointClique;
            }
            // Si c'est le deuxième point et que le premier était un point de contrôle,
            // créer une ligne droite
            else if (pointsCliques.size() == 2 && pointSelectionne != null) {
                // Créer une ligne droite
                try {
                    ajouter(pointSelectionne, pointClique);
                } catch (ArcException e) {
                    // Réinitialiser en cas d'erreur
                    pointSelectionne = null;
                    pointsCliques.clear();
                    throw e;
                }

                // Réinitialiser
                pointSelectionne = null;
                pointsCliques.clear();
            }
            // Si c'est le quatrième point et que le premier était un point de contrôle,
            // créer une courbe
            else if (pointsCliques.size() == NB_POINTS_COURBE && pointSelectionne != null) {
                // Récupérer les points intermédiaires
                Point2D p2 = pointsCliques.get(1);
                Point2D p3 = pointsCliques.get(2);

                // Vérifier les contraintes comme dans la méthode ajouter
                if (pointSelectionne == pointClique) {
                    pointSelectionne = null;
                    pointsCliques.clear();
                    throw new ArcException("Impossible de créer un arc: les deux points sont identiques");
                }

                if (pointSelectionne.getEtape().equals(pointClique.getEtape())) {
                    pointSelectionne = null;
                    pointsCliques.clear();
                    throw new ArcException("Impossible de créer un arc: les deux points appartiennent à la même étape");
                }

                // Vérifier les autres contraintes comme dans la méthode ajouter
                for (ArcIG arc : arcs) {
                    if ((arc.getP1().equals(pointSelectionne) && arc.getP2().equals(pointClique))) {
                        pointSelectionne = null;
                        pointsCliques.clear();
                        throw new ArcException("Impossible de créer un arc: un arc existe déjà entre ces deux points");
                    }
                }

                for (ArcIG arc : arcs) {
                    if ((arc.getP1().equals(pointClique) && arc.getP2().equals(pointSelectionne)) ||
                            (arc.getP1().getEtape().equals(pointClique.getEtape()) && arc.getP2().getEtape().equals(pointSelectionne.getEtape()))) {
                        pointSelectionne = null;
                        pointsCliques.clear();
                        throw new ArcException("Impossible de créer un arc: un arc existe déjà dans le sens inverse");
                    }
                }

                for (ArcIG arc : arcs) {
                    if (arc.getP1().equals(pointSelectionne)) {
                        pointSelectionne = null;
                        pointsCliques.clear();
                        throw new ArcException("Impossible de créer un arc: le point de départ est déjà utilisé comme source d'un autre arc");
                    }
                }

                for (ArcIG arc : arcs) {
                    if (arc.getP2().equals(pointClique)) {
                        pointSelectionne = null;
                        pointsCliques.clear();
                        throw new ArcException("Impossible de créer un arc: le point d'arrivée est déjà utilisé comme destination d'un autre arc");
                    }
                }

                // Créer une courbe
                ArcIG arc = new CourbeIG(
                        pointSelectionne,
                        pointClique,
                        (int) p2.getX(), (int) p2.getY(),
                        (int) p3.getX(), (int) p3.getY()
                );
                arcs.add(arc);

                // Réinitialiser
                pointSelectionne = null;
                pointsCliques.clear();

                // Notifier les observateurs
                notifierObservateurs();
            }
        }
        // Sinon, si on est en train de créer une courbe (entre 1 et 3 points déjà cliqués),
        // ajouter le point comme point de contrôle intermédiaire
        else if (pointsCliques.size() >= 1 && pointsCliques.size() < NB_POINTS_COURBE - 1) {
            pointsCliques.add(new Point2D(x, y));
        }
    }
    /**
     * Gère la sélection/désélection d'une étape
     * @param etape l'étape à sélectionner/désélectionner
     */
    public void selectionnerEtape(EtapeIG etape) {
        String id = etape.getIdentifiant();

        // Si l'étape est déjà sélectionnée, la désélectionner
        if (etapesSelectionnees.contains(id)) {
            etapesSelectionnees.remove(id);
        } else {
            // Sinon, la sélectionner
            etapesSelectionnees.add(id);
        }

        // Notifier les observateurs pour mettre à jour l'affichage
        notifierObservateurs();
    }

    /**
     * Vérifie si une étape est sélectionnée
     * @param etape l'étape à vérifier
     * @return true si l'étape est sélectionnée, false sinon
     */
    public boolean estSelectionnee(EtapeIG etape) {
        return etapesSelectionnees.contains(etape.getIdentifiant());
    }

    /**
     * Retourne l'ensemble des identifiants des étapes sélectionnées
     * @return ensemble des identifiants
     */
    public Set<String> getEtapesSelectionnees() {
        return Collections.unmodifiableSet(etapesSelectionnees);
    }

    /**
     * Supprime toutes les étapes sélectionnées et les arcs qui leur sont attachés
     */
    public void supprimerEtapesSelectionnees() {
        // Créer une liste temporaire pour éviter les problèmes de concurrence
        List<String> aSupprimer = new ArrayList<>(etapesSelectionnees);

        // Supprimer les arcs attachés aux étapes sélectionnées
        Iterator<ArcIG> itArc = arcs.iterator();
        while (itArc.hasNext()) {
            ArcIG arc = itArc.next();
            // Si l'arc est attaché à une étape sélectionnée, le supprimer
            if (etapesSelectionnees.contains(arc.getP1().getEtape().getIdentifiant()) ||
                    etapesSelectionnees.contains(arc.getP2().getEtape().getIdentifiant())) {
                itArc.remove();
            }
        }

        // Supprimer les étapes sélectionnées
        for (String id : aSupprimer) {
            etapes.remove(id);
        }

        // Vider la liste des étapes sélectionnées
        etapesSelectionnees.clear();

        // Notifier les observateurs
        notifierObservateurs();
    }
    /**
     * Retourne le nombre d'étapes sélectionnées
     * @return nombre d'étapes sélectionnées
     */
    public int getNbEtapesSelectionnees() {
        return etapesSelectionnees.size();
    }

    /**
     * Retourne l'étape sélectionnée (si une seule est sélectionnée)
     * @return l'étape sélectionnée ou null si aucune ou plusieurs sont sélectionnées
     */
    public EtapeIG getEtapeSelectionnee() {
        if (etapesSelectionnees.size() != 1) {
            return null;
        }

        // Récupérer l'identifiant de l'étape sélectionnée
        String id = etapesSelectionnees.iterator().next();

        // Retourner l'étape correspondante
        return etapes.get(id);
    }

    /**
     * Renomme une étape
     * @param etape l'étape à renommer
     * @param nouveauNom le nouveau nom de l'étape
     */
    public void renommerEtape(EtapeIG etape, String nouveauNom) {
        // Vérifier que l'étape existe
        if (etapes.containsKey(etape.getIdentifiant())) {
            // Renommer l'étape
            etape.setNom(nouveauNom);

            // Notifier les observateurs
            notifierObservateurs();
        }
    }

    /**
     * Déplace une étape aux coordonnées spécifiées
     * @param idEtape l'identifiant de l'étape à déplacer
     * @param x la nouvelle coordonnée x
     * @param y la nouvelle coordonnée y
     */
    public void deplacerEtape(String idEtape, int x, int y) {
        // Vérifier que l'étape existe
        if (etapes.containsKey(idEtape)) {
            EtapeIG etape = etapes.get(idEtape);

            // Calculer les nouvelles coordonnées en tenant compte du centre de l'étape
            int newX = x - etape.getLargeur() / 2;
            int newY = y - etape.getHauteur() / 2;

            // Éviter les coordonnées négatives
            newX = Math.max(0, newX);
            newY = Math.max(0, newY);

            // Mettre à jour les coordonnées de l'étape
            etape.setPosX(newX);
            etape.setPosY(newY);

            // Notifier les observateurs
            notifierObservateurs();
        }
    }
    /**
     * Sélectionne ou désélectionne un arc
     * @param arc l'arc à sélectionner/désélectionner
     */
    public void selectionnerArc(ArcIG arc) {
        if (arcsSelectionnes.contains(arc)) {
            // Si l'arc est déjà sélectionné, le désélectionner
            arcsSelectionnes.remove(arc);
        } else {
            // Sinon, le sélectionner
            arcsSelectionnes.add(arc);
        }

        // Notifier les observateurs pour mettre à jour l'affichage
        notifierObservateurs();
    }


    /**
     * Vérifie si un arc est sélectionné
     * @param arc l'arc à vérifier
     * @return true si l'arc est sélectionné, false sinon
     */
    public boolean estSelectionne(ArcIG arc) {
        return arcsSelectionnes.contains(arc);
    }

    /**
     * Supprime toutes les étapes sélectionnées et les arcs sélectionnés
     */
    public void supprimerSelection() {
        // Créer une liste temporaire pour éviter les ConcurrentModificationException
        List<ArcIG> arcsASupprimer = new ArrayList<>();

        // Supprimer les arcs attachés aux étapes sélectionnées
        for (ArcIG arc : arcs) {
            if (etapesSelectionnees.contains(arc.getP1().getEtape().getIdentifiant()) ||
                    etapesSelectionnees.contains(arc.getP2().getEtape().getIdentifiant())) {
                arcsASupprimer.add(arc);
            }
        }

        // Supprimer les arcs sélectionnés
        for (ArcIG arc : arcsSelectionnes) {
            if (!arcsASupprimer.contains(arc)) {
                arcsASupprimer.add(arc);
            }
        }

        // Supprimer les arcs de la liste principale
        arcs.removeAll(arcsASupprimer);

        // Supprimer les étapes sélectionnées
        for (String id : new ArrayList<>(etapesSelectionnees)) {
            etapes.remove(id);
        }

        // Vider les listes de sélection
        etapesSelectionnees.clear();
        arcsSelectionnes.clear();

        // Notifier les observateurs
        notifierObservateurs();
    }
    /**
     * Désélectionne tous les éléments sélectionnés (étapes et arcs)
     */
    public void deselectionnerTout() {
        // Vider les ensembles de sélection
        etapesSelectionnees.clear();
        arcsSelectionnes.clear();

        // Notifier les observateurs
        notifierObservateurs();
    }
    /**
     * Retourne un point de contrôle à partir de son ID
     * @param id l'ID du point de contrôle
     * @return le point de contrôle ou null s'il n'existe pas
     */
    public PointDeControleIG getPointDeControleById(String id) {
        // Parcourir toutes les étapes
        for (EtapeIG etape : etapes.values()) {
            // Parcourir tous les points de contrôle de l'étape
            for (PointDeControleIG pt : etape) {
                if (pt.getId().equals(id)) {
                    return pt;
                }
            }
        }
        return null;
    }
    /**
     * Gère le drag and drop entre deux points de contrôle
     * @param pointSource le point de contrôle source
     * @param pointCible le point de contrôle cible
     * @return true si l'opération a réussi, false sinon avec un message d'erreur
     */
    public Pair<Boolean, String> gererDragAndDropPoints(PointDeControleIG pointSource, PointDeControleIG pointCible) {
        try {
            // Vérifier que les points sont différents
            if (pointSource == pointCible) {
                return new Pair<>(false, "Impossible de créer un arc: les deux points sont identiques");
            }

            // Vérifier que les points ne sont pas sur la même étape
            if (pointSource.getEtape().equals(pointCible.getEtape())) {
                return new Pair<>(false, "Impossible de créer un arc: les deux points appartiennent à la même étape");
            }

            // Vérifier qu'il n'existe pas déjà un arc entre ces deux points
            for (ArcIG arc : arcs) {
                if ((arc.getP1().equals(pointSource) && arc.getP2().equals(pointCible))) {
                    return new Pair<>(false, "Impossible de créer un arc: un arc existe déjà entre ces deux points");
                }
            }

            // Vérifier qu'il n'existe pas déjà un arc dans le sens inverse
            for (ArcIG arc : arcs) {
                if ((arc.getP1().equals(pointCible) && arc.getP2().equals(pointSource)) ||
                        (arc.getP1().getEtape().equals(pointCible.getEtape()) && arc.getP2().getEtape().equals(pointSource.getEtape()))) {
                    return new Pair<>(false, "Impossible de créer un arc: un arc existe déjà dans le sens inverse");
                }
            }

            // Vérifier que le point de départ n'est pas déjà utilisé comme source d'un autre arc
            for (ArcIG arc : arcs) {
                if (arc.getP1().equals(pointSource)) {
                    return new Pair<>(false, "Impossible de créer un arc: le point de départ est déjà utilisé comme source d'un autre arc");
                }
            }

            // Vérifier que le point d'arrivée n'est pas déjà utilisé comme destination d'un autre arc
            for (ArcIG arc : arcs) {
                if (arc.getP2().equals(pointCible)) {
                    return new Pair<>(false, "Impossible de créer un arc: le point d'arrivée est déjà utilisé comme destination d'un autre arc");
                }
            }

            // Toutes les contraintes sont satisfaites, on peut ajouter l'arc
            ajouter(pointSource, pointCible);
            return new Pair<>(true, "");
        } catch (ArcException e) {
            return new Pair<>(false, e.getMessage());
        }
    }
    /**
     * Définit ou supprime les étapes sélectionnées comme entrées
     */
    public void definirEntrees() {
        // Pour chaque étape sélectionnée
        for (String idEtape : etapesSelectionnees) {
            // Si l'étape est déjà une entrée, la supprimer
            if (entrees.contains(idEtape)) {
                entrees.remove(idEtape);
            } else {
                // Sinon, l'ajouter comme entrée et la retirer des sorties si elle y est
                entrees.add(idEtape);
                // Une entrée ne peut pas être une sortie
                sorties.remove(idEtape);
            }
        }

        // Notifier les observateurs pour mettre à jour l'affichage
        notifierObservateurs();
    }

    /**
     * Définit ou supprime les étapes sélectionnées comme sorties
     */
    public void definirSorties() {
        // Pour chaque étape sélectionnée
        for (String idEtape : etapesSelectionnees) {
            // Si l'étape est déjà une sortie, la supprimer
            if (sorties.contains(idEtape)) {
                sorties.remove(idEtape);
            } else {
                // Sinon, l'ajouter comme sortie et la retirer des entrées si elle y est
                sorties.add(idEtape);
                // Une sortie ne peut pas être une entrée
                entrees.remove(idEtape);
            }
        }

        // Notifier les observateurs pour mettre à jour l'affichage
        notifierObservateurs();
    }

    /**
     * Vérifie si une étape est une entrée
     * @param idEtape l'identifiant de l'étape
     * @return true si l'étape est une entrée, false sinon
     */
    public boolean estUneEntree(String idEtape) {
        return entrees.contains(idEtape);
    }

    /**
     * Vérifie si une étape est une sortie
     * @param idEtape l'identifiant de l'étape
     * @return true si l'étape est une sortie, false sinon
     */
    public boolean estUneSortie(String idEtape) {
        return sorties.contains(idEtape);
    }

    /**
     * Définit les paramètres (délai et écart) de l'activité sélectionnée
     * @param delai le délai en unités de temps
     * @param ecart l'écart en unités de temps
     * @throws ParametresException si les paramètres sont incorrects
     * @throws IllegalStateException si aucune activité n'est sélectionnée ou si plusieurs étapes sont sélectionnées
     */
    public void definirParametres(int delai, int ecart) throws ParametresException, IllegalStateException {
        // Vérifier qu'une seule étape est sélectionnée
        if (etapesSelectionnees.size() != 1) {
            throw new IllegalStateException("Il faut sélectionner une seule activité pour définir ses paramètres");
        }

        // Récupérer l'étape sélectionnée
        String idEtape = etapesSelectionnees.iterator().next();
        EtapeIG etape = etapes.get(idEtape);

        // Vérifier que l'étape est bien une activité
        if (!(etape instanceof ActiviteIG)) {
            throw new IllegalStateException("Seules les activités peuvent avoir des paramètres de délai et d'écart");
        }

        // Définir les paramètres
        ActiviteIG activite = (ActiviteIG) etape;
        activite.setDelai(delai);
        activite.setEcart(ecart);

        // Notifier les observateurs
        notifierObservateurs();
    }

    /**
     * Vérifie si une seule activité est sélectionnée
     * @return true si une seule activité est sélectionnée, false sinon
     */
    public boolean uneSeuleActiviteSelectionnee() {
        if (etapesSelectionnees.size() != 1) {
            return false;
        }

        String idEtape = etapesSelectionnees.iterator().next();
        EtapeIG etape = etapes.get(idEtape);

        return etape instanceof ActiviteIG;
    }

    /**
     * Retourne l'activité sélectionnée si une seule activité est sélectionnée
     * @return l'activité sélectionnée
     * @throws IllegalStateException si aucune activité n'est sélectionnée ou si plusieurs étapes sont sélectionnées
     */
    public ActiviteIG getActiviteSelectionnee() throws IllegalStateException {
        if (etapesSelectionnees.size() != 1) {
            throw new IllegalStateException("Il faut sélectionner une seule activité");
        }

        String idEtape = etapesSelectionnees.iterator().next();
        EtapeIG etape = etapes.get(idEtape);

        if (!(etape instanceof ActiviteIG)) {
            throw new IllegalStateException("L'étape sélectionnée n'est pas une activité");
        }

        return (ActiviteIG) etape;
    }
    /**
     * Vérifie si l'ajout d'un arc entre deux étapes créerait un cycle dans le graphe
     * @param etapeSource l'étape source
     * @param etapeDestination l'étape destination
     * @return true si l'ajout créerait un cycle, false sinon
     */
    private boolean creeraitUnCycle(EtapeIG etapeSource, EtapeIG etapeDestination) {
        // Si on peut atteindre l'étape source à partir de l'étape destination,
        // alors ajouter un arc de source vers destination créerait un cycle
        return peutAtteindre(etapeDestination, etapeSource);
    }

    /**
     * Vérifie si on peut atteindre une étape cible à partir d'une étape source
     * en suivant les arcs existants
     * @param etapeSource l'étape source
     * @param etapeCible l'étape cible
     * @return true si on peut atteindre l'étape cible, false sinon
     */
    private boolean peutAtteindre(EtapeIG etapeSource, EtapeIG etapeCible) {
        // Ensemble des étapes déjà visitées pour éviter les boucles infinies
        Set<String> etapesVisitees = new HashSet<>();

        // File pour le parcours en largeur
        Queue<EtapeIG> file = new LinkedList<>();
        file.add(etapeSource);

        while (!file.isEmpty()) {
            EtapeIG etapeCourante = file.poll();

            // Si on a atteint l'étape cible, retourner true
            if (etapeCourante.equals(etapeCible)) {
                return true;
            }

            // Marquer l'étape comme visitée
            etapesVisitees.add(etapeCourante.getIdentifiant());

            // Ajouter toutes les étapes successeurs à la file
            for (ArcIG arc : arcs) {
                if (arc.getP1().getEtape().equals(etapeCourante)) {
                    EtapeIG etapeSuccesseur = arc.getP2().getEtape();
                    if (!etapesVisitees.contains(etapeSuccesseur.getIdentifiant())) {
                        file.add(etapeSuccesseur);
                    }
                }
            }
        }

        // Si on a parcouru tout le graphe sans trouver l'étape cible, retourner false
        return false;
    }


}
