package twisk.test.modele;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import twisk.exceptions.ArcException;
import twisk.modele.*;

import java.util.Iterator;

import static org.junit.jupiter.api.Assertions.*;

class MondeIGTest {

    @BeforeEach
    void setUp() {
        FabriqueIdentifiant.getInstance().reset();
    }

    @Test
    void testConstructeur() {
        MondeIG monde = new MondeIG();

        // Vérifier qu'il y a une activité par défaut
        Iterator<EtapeIG> it = monde.iterator();
        assertTrue(it.hasNext(), "Le monde devrait contenir au moins une étape");

        EtapeIG etape = it.next();
        assertTrue(etape instanceof ActiviteIG, "L'étape devrait être une ActiviteIG");
        assertEquals("Activite 1", etape.getNom(), "Le nom par défaut devrait être 'Activite 1'");
        assertFalse(it.hasNext(), "Le monde ne devrait contenir qu'une seule étape à l'initialisation");
    }

    @Test
    void testAjouter() {
        MondeIG monde = new MondeIG();

        // Ajouter une nouvelle activité
        monde.ajouter("Activite");

        // Vérifier qu'il y a maintenant deux activités
        Iterator<EtapeIG> it = monde.iterator();
        assertTrue(it.hasNext());
        EtapeIG etape1 = it.next();
        assertTrue(it.hasNext());
        EtapeIG etape2 = it.next();
        assertFalse(it.hasNext());

        // Vérifier que les noms sont différents
        assertNotEquals(etape1.getNom(), etape2.getNom(), "Les noms des activités devraient être différents");

        // Vérifier que les identifiants sont différents
        assertNotEquals(etape1.getIdentifiant(), etape2.getIdentifiant(), "Les identifiants devraient être différents");
    }

    @Test
    void testIterator() {
        MondeIG monde = new MondeIG();
        monde.ajouter("Activite");
        monde.ajouter("Activite");

        int count = 0;
        for (EtapeIG etape : monde) {
            assertNotNull(etape, "L'étape ne devrait pas être null");
            assertTrue(etape instanceof ActiviteIG, "L'étape devrait être une ActiviteIG");
            count++;
        }

        assertEquals(3, count, "Le monde devrait contenir 3 étapes");
    }
    @Test
    void testAjouterArcAvecExceptions() {
        MondeIG monde = new MondeIG();

        // Ajouter deux activités
        monde.ajouter("Activite"); // Activite 2

        // Récupérer les activités
        Iterator<EtapeIG> it = monde.iterator();
        EtapeIG etape1 = it.next(); // Activite 1 (par défaut)
        EtapeIG etape2 = it.next(); // Activite 2

        // Récupérer les points de contrôle
        Iterator<PointDeControleIG> itPt1 = etape1.iterator();
        PointDeControleIG pt1 = itPt1.next(); // Premier point de l'activité 1

        Iterator<PointDeControleIG> itPt2 = etape2.iterator();
        PointDeControleIG pt2 = itPt2.next(); // Premier point de l'activité 2

        // Test d'ajout d'un arc valide
        try {
            monde.ajouter(pt1, pt2);
            // Vérifier que l'arc a été ajouté
            Iterator<ArcIG> itArc = monde.iteratorArcs();
            assertTrue(itArc.hasNext(), "Un arc devrait avoir été ajouté");
        } catch (ArcException e) {
            fail("L'ajout d'un arc valide ne devrait pas lever d'exception");
        }

        // Test de la contrainte 2: pas de duplication d'arc
        try {
            monde.ajouter(pt1, pt2);
            fail("Une exception aurait dû être levée pour un arc dupliqué");
        } catch (ArcException e) {
            // C'est le comportement attendu
        }

        // Test de la contrainte 3: pas d'arc dans le sens inverse
        try {
            monde.ajouter(pt2, pt1);
            fail("Une exception aurait dû être levée pour un arc dans le sens inverse");
        } catch (ArcException e) {
            // C'est le comportement attendu
        }

        // Test de la contrainte 5: un point ne peut être source que d'un seul arc
        PointDeControleIG pt1bis = itPt1.next(); // Deuxième point de l'activité 1
        PointDeControleIG pt2bis = itPt2.next(); // Deuxième point de l'activité 2
        try {
            monde.ajouter(pt1, pt2bis);
            fail("Une exception aurait dû être levée pour un point source déjà utilisé");
        } catch (ArcException e) {
            // C'est le comportement attendu
        }

        // Test de la contrainte 6: un point ne peut être destination que d'un seul arc
        try {
            monde.ajouter(pt1bis, pt2);
            fail("Une exception aurait dû être levée pour un point destination déjà utilisé");
        } catch (ArcException e) {
            // C'est le comportement attendu
        }

        // Test de la contrainte 1: pas d'arc entre points de la même étape
        PointDeControleIG pt1ter = itPt1.next(); // Troisième point de l'activité 1
        try {
            monde.ajouter(pt1, pt1ter);
            fail("Une exception aurait dû être levée pour un arc entre points de la même étape");
        } catch (ArcException e) {
            // C'est le comportement attendu
        }
    }

}
