package twisk.vues;

import javafx.application.Platform;
import javafx.beans.binding.Bindings;
import javafx.beans.property.BooleanProperty;
import javafx.beans.property.SimpleBooleanProperty;
import javafx.geometry.Insets;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.input.KeyCombination;
import javafx.scene.layout.GridPane;
import javafx.util.Pair;
import twisk.exceptions.ParametresException;
import twisk.modele.ActiviteIG;
import twisk.modele.MondeIG;

/**
 * Vue représentant la barre de menu de l'application
 */
public class VueMenu extends MenuBar {
    private final MondeIG monde;
    private final Scene scene;
    private final BooleanProperty uneEtapeSelectionnee;
    private final BooleanProperty plusieursEtapesSelectionnees;
    private final BooleanProperty uneActiviteSelectionnee;

    /**
     * Constructeur
     * @param monde le monde à manipuler
     * @param scene la scène principale de l'application
     */
    public VueMenu(MondeIG monde, Scene scene) {
        this.monde = monde;
        this.scene = scene;

        // Propriétés pour le binding
        uneEtapeSelectionnee = new SimpleBooleanProperty(false);
        plusieursEtapesSelectionnees = new SimpleBooleanProperty(false);
        uneActiviteSelectionnee = new SimpleBooleanProperty(false);

        // Création du menu Fichier
        Menu menuFichier = new Menu("Fichier");

        // Création de l'item Quitter
        MenuItem itemQuitter = new MenuItem("Quitter");
        itemQuitter.setAccelerator(KeyCombination.keyCombination("Ctrl+Q"));
        itemQuitter.setOnAction(event -> Platform.exit());

        // Ajout de l'item au menu Fichier
        menuFichier.getItems().add(itemQuitter);

        // Création du menu Édition
        Menu menuEdition = new Menu("Édition");

        // Création de l'item Supprimer
        MenuItem itemSupprimer = new MenuItem("Supprimer");
        itemSupprimer.setAccelerator(KeyCombination.keyCombination("Delete"));
        itemSupprimer.setOnAction(event -> monde.supprimerEtapesSelectionnees());
        // Activer seulement si au moins une étape est sélectionnée
        itemSupprimer.disableProperty().bind(
                uneEtapeSelectionnee.not().and(plusieursEtapesSelectionnees.not())
        );

        // Création de l'item Renommer
        MenuItem itemRenommer = new MenuItem("Renommer");
        itemRenommer.setAccelerator(KeyCombination.keyCombination("Ctrl+R"));
        itemRenommer.setOnAction(event -> afficherDialogueRenommer());
        // Activer seulement si une seule étape est sélectionnée
        itemRenommer.disableProperty().bind(uneEtapeSelectionnee.not());

        // Création de l'item Désélectionner
        MenuItem itemDeselectionner = new MenuItem("Désélectionner tout");
        itemDeselectionner.setAccelerator(KeyCombination.keyCombination("Escape"));
        itemDeselectionner.setOnAction(event -> monde.deselectionnerTout());
        // Activer seulement si au moins une étape est sélectionnée
        itemDeselectionner.disableProperty().bind(
                uneEtapeSelectionnee.not().and(plusieursEtapesSelectionnees.not())
        );

        // Ajout des items au menu Édition
        menuEdition.getItems().addAll(itemSupprimer, itemRenommer, new SeparatorMenuItem(), itemDeselectionner);

        // Création du menu Paramètres
        Menu menuParametres = new Menu("Paramètres");

        // Création de l'item Paramètres
        MenuItem itemParametres = new MenuItem("Définir délai et écart");
        itemParametres.setOnAction(event -> afficherDialogueParametres());
        // Activer seulement si une seule activité est sélectionnée
        itemParametres.disableProperty().bind(uneActiviteSelectionnee.not());

        // Ajout de l'item au menu Paramètres
        menuParametres.getItems().add(itemParametres);

        // Création du menu Monde
        Menu menuMonde = new Menu("Monde");

        // Création de l'item Entrée
        MenuItem itemEntree = new MenuItem("Entrée");
        itemEntree.setOnAction(event -> monde.definirEntrees());
        // Activer seulement si au moins une étape est sélectionnée
        itemEntree.disableProperty().bind(
                uneEtapeSelectionnee.not().and(plusieursEtapesSelectionnees.not())
        );

        // Création de l'item Sortie
        MenuItem itemSortie = new MenuItem("Sortie");
        itemSortie.setOnAction(event -> monde.definirSorties());
        // Activer seulement si au moins une étape est sélectionnée
        itemSortie.disableProperty().bind(
                uneEtapeSelectionnee.not().and(plusieursEtapesSelectionnees.not())
        );

        // Ajout des items au menu Monde
        menuMonde.getItems().addAll(itemEntree, itemSortie);

        // Création du menu Style
        Menu menuStyle = new Menu("Style");

        // Créer un groupe pour les styles (pour que seul un style soit actif à la fois)
        ToggleGroup groupeStyles = new ToggleGroup();

        // Créer les items de style
        RadioMenuItem styleClassique = new RadioMenuItem("Classique");
        styleClassique.setToggleGroup(groupeStyles);
        styleClassique.setSelected(true); // Style par défaut
        styleClassique.setOnAction(event -> changerStyle("style"));

        RadioMenuItem styleSombre = new RadioMenuItem("Sombre");
        styleSombre.setToggleGroup(groupeStyles);
        styleSombre.setOnAction(event -> changerStyle("style-sombre"));

        RadioMenuItem styleColoré = new RadioMenuItem("Coloré");
        styleColoré.setToggleGroup(groupeStyles);
        styleColoré.setOnAction(event -> changerStyle("style-colore"));

        RadioMenuItem styleMinimaliste = new RadioMenuItem("Minimaliste");
        styleMinimaliste.setToggleGroup(groupeStyles);
        styleMinimaliste.setOnAction(event -> changerStyle("style-minimaliste"));

        // Ajouter les items au menu Style
        menuStyle.getItems().addAll(styleClassique, styleSombre, styleColoré, styleMinimaliste);

        // Ajout des menus à la barre de menu
        this.getMenus().addAll(menuFichier, menuEdition, menuParametres, menuMonde, menuStyle);

        // S'enregistrer comme observateur du monde pour mettre à jour l'état du menu
        monde.ajouterObservateur(() -> mettreAJourMenu());
    }

    /**
     * Met à jour l'état du menu en fonction de l'état du monde
     */
    private void mettreAJourMenu() {
        int nbEtapesSelectionnees = monde.getNbEtapesSelectionnees();
        uneEtapeSelectionnee.set(nbEtapesSelectionnees == 1);
        plusieursEtapesSelectionnees.set(nbEtapesSelectionnees > 1);
        uneActiviteSelectionnee.set(monde.uneSeuleActiviteSelectionnee());
    }

    /**
     * Affiche une boîte de dialogue pour renommer l'étape sélectionnée
     */
    private void afficherDialogueRenommer() {
        // Récupérer l'étape sélectionnée
        if (monde.getNbEtapesSelectionnees() != 1) return;

        // Créer une boîte de dialogue
        TextInputDialog dialog = new TextInputDialog(monde.getEtapeSelectionnee().getNom());
        dialog.setTitle("Renommer l'activité");
        dialog.setHeaderText("Entrez le nouveau nom de l'activité");
        dialog.setContentText("Nom:");

        // Récupérer le résultat
        dialog.showAndWait().ifPresent(nouveauNom -> {
            if (!nouveauNom.isEmpty()) {
                monde.renommerEtape(monde.getEtapeSelectionnee(), nouveauNom);
            }
        });
    }

    /**
     * Affiche une boîte de dialogue pour saisir les paramètres de l'activité sélectionnée
     */
    private void afficherDialogueParametres() {
        // Vérifier qu'une seule activité est sélectionnée
        if (!monde.uneSeuleActiviteSelectionnee()) return;

        try {
            // Récupérer l'activité sélectionnée
            ActiviteIG activite = monde.getActiviteSelectionnee();

            // Créer une boîte de dialogue personnalisée
            Dialog<Pair<Integer, Integer>> dialog = new Dialog<>();
            dialog.setTitle("Paramètres de l'activité");
            dialog.setHeaderText("Définir le délai et l'écart");

            // Ajouter les boutons
            ButtonType boutonValider = new ButtonType("Valider", ButtonBar.ButtonData.OK_DONE);
            dialog.getDialogPane().getButtonTypes().addAll(boutonValider, ButtonType.CANCEL);

            // Créer la grille pour les champs
            GridPane grid = new GridPane();
            grid.setHgap(10);
            grid.setVgap(10);
            grid.setPadding(new Insets(20, 150, 10, 10));

            // Créer les champs
            TextField champDelai = new TextField();
            champDelai.setPromptText("Délai");
            TextField champEcart = new TextField();
            champEcart.setPromptText("Écart");

            // Récupérer les valeurs actuelles
            champDelai.setText(String.valueOf(activite.getDelai()));
            champEcart.setText(String.valueOf(activite.getEcart()));

            // Ajouter les champs à la grille
            grid.add(new Label("Délai:"), 0, 0);
            grid.add(champDelai, 1, 0);
            grid.add(new Label("Écart:"), 0, 1);
            grid.add(champEcart, 1, 1);

            // Ajouter la grille à la boîte de dialogue
            dialog.getDialogPane().setContent(grid);

            // Convertir le résultat
            dialog.setResultConverter(dialogButton -> {
                if (dialogButton == boutonValider) {
                    try {
                        int delai = Integer.parseInt(champDelai.getText());
                        int ecart = Integer.parseInt(champEcart.getText());
                        return new Pair<>(delai, ecart);
                    } catch (NumberFormatException e) {
                        return null;
                    }
                }
                return null;
            });

            // Afficher la boîte de dialogue et traiter le résultat
            dialog.showAndWait().ifPresent(parametres -> {
                try {
                    monde.definirParametres(parametres.getKey(), parametres.getValue());
                } catch (ParametresException e) {
                    // Afficher un message d'erreur
                    Alert alert = new Alert(Alert.AlertType.ERROR);
                    alert.setTitle("Erreur");
                    alert.setHeaderText("Paramètres invalides");
                    alert.setContentText(e.getMessage());
                    alert.showAndWait();
                } catch (IllegalStateException e) {
                    // Afficher un message d'erreur
                    Alert alert = new Alert(Alert.AlertType.ERROR);
                    alert.setTitle("Erreur");
                    alert.setHeaderText("Sélection invalide");
                    alert.setContentText(e.getMessage());
                    alert.showAndWait();
                }
            });
        } catch (IllegalStateException e) {
            // Afficher un message d'erreur
            Alert alert = new Alert(Alert.AlertType.ERROR);
            alert.setTitle("Erreur");
            alert.setHeaderText("Sélection invalide");
            alert.setContentText(e.getMessage());
            alert.showAndWait();
        }
    }

    /**
     * Change le style de l'application
     * @param nomStyle le nom du fichier CSS de style (sans l'extension)
     */
    private void changerStyle(String nomStyle) {
        // Supprimer tous les styles existants
        scene.getStylesheets().clear();

        // Ajouter le nouveau style
        String cssPath = getClass().getResource("/twisk/ressources/css/" + nomStyle + ".css").toExternalForm();
        scene.getStylesheets().add(cssPath);
    }
}
