package twisk.vues;

import javafx.animation.PauseTransition;
import javafx.scene.control.Alert;
import javafx.scene.input.DragEvent;
import javafx.scene.input.Dragboard;
import javafx.scene.input.TransferMode;
import javafx.scene.layout.Pane;
import javafx.util.Duration;
import twisk.exceptions.ArcException;
import twisk.modele.*;
import twisk.outils.Observateur;

import java.util.Iterator;

public class VueMondeIG extends Pane implements Observateur {
    private MondeIG monde;

    public VueMondeIG(MondeIG monde) {
        this.monde = monde;

        // Appliquer la classe CSS
        this.getStyleClass().add("monde");

        // S'enregistrer comme observateur du monde
        monde.ajouterObservateur(this);

        // Ajouter un écouteur de clic sur le monde
        this.setOnMouseClicked(event -> {
            try {
                monde.clic(event.getX(), event.getY());
            } catch (ArcException e) {
                afficherMessageErreur(e.getMessage());
            }
        });

        // Ajouter les écouteurs pour le drag and drop
        this.setOnDragOver(this::gererDragOver);
        this.setOnDragDropped(this::gererDragDropped);

        // Initialiser la vue
        reagir();
    }

    /**
     * Gère l'événement de survol pendant le drag
     * @param event l'événement de drag
     */
    private void gererDragOver(DragEvent event) {
        // Accepter uniquement si c'est un drag de type MOVE et s'il contient une chaîne
        if (event.getGestureSource() != this &&
                event.getDragboard().hasString() &&
                event.getDragboard().hasImage()) {

            // Accepter le transfert
            event.acceptTransferModes(TransferMode.MOVE);
        }

        // Consommer l'événement
        event.consume();
    }

    /**
     * Gère l'événement de drop
     * @param event l'événement de drag
     */
    private void gererDragDropped(DragEvent event) {
        // Récupérer le dragboard
        Dragboard db = event.getDragboard();
        boolean success = false;

        // Vérifier s'il contient une chaîne (identifiant de l'étape)
        if (db.hasString()) {
            // Récupérer l'identifiant de l'étape
            String idEtape = db.getString();

            // Récupérer les coordonnées de la souris
            double x = event.getX();
            double y = event.getY();

            // Afficher les informations pour vérification
            System.out.println("Drop de l'étape: " + idEtape + " aux coordonnées: " + x + ", " + y);

            // Déplacer l'étape dans le modèle
            monde.deplacerEtape(idEtape, (int) x, (int) y);

            // Indiquer que le transfert a réussi
            success = true;
        }

        // Indiquer si le transfert a réussi ou échoué
        event.setDropCompleted(success);

        // Consommer l'événement
        event.consume();
    }

    @Override
    public void reagir() {
        // Effacer tous les composants existants
        this.getChildren().clear();

        // D'abord, afficher les étapes
        for (EtapeIG etape : monde) {
            // Créer et ajouter la vue de l'étape
            VueEtapeIG vueEtape = new VueActiviteIG(etape, monde);
            this.getChildren().add(vueEtape);
        }

        // Ensuite, afficher les arcs (pour qu'ils soient au-dessus des étapes)
        Iterator<ArcIG> itArcs = monde.iteratorArcs();
        while (itArcs.hasNext()) {
            ArcIG arc = itArcs.next();
            VueArcIG vueArc;

            // Créer la vue appropriée selon le type d'arc
            if (arc instanceof LigneDroiteIG) {
                vueArc = new VueLigneDroiteIG(arc, monde);
            } else if (arc instanceof CourbeIG) {
                vueArc = new VueCourbeIG(arc, monde);
            } else {
                // Ne devrait jamais arriver, mais au cas où
                continue;
            }

            // Mettre à jour le style en fonction de l'état de sélection
            vueArc.mettreAJourStyle();

            this.getChildren().add(vueArc);
        }

        // Enfin, afficher les points de contrôle (pour qu'ils soient tout en haut)
        for (EtapeIG etape : monde) {
            for (PointDeControleIG pt : etape) {
                VuePointDeControleIG vuePt = new VuePointDeControleIG(pt, monde);
                this.getChildren().add(vuePt);
            }
        }
    }



    /**
     * Affiche un message d'erreur dans une boîte de dialogue
     * @param message le message à afficher
     */
    private void afficherMessageErreur(String message) {
        // Créer une boîte de dialogue d'alerte
        Alert alert = new Alert(Alert.AlertType.ERROR);
        alert.setTitle("Erreur");
        alert.setHeaderText("Impossible de créer l'arc");
        alert.setContentText(message);

        // Configurer la fermeture automatique après 3 secondes
        PauseTransition delay = new PauseTransition(Duration.seconds(3));
        delay.setOnFinished(event -> alert.close());
        delay.play();

        // Afficher la boîte de dialogue
        alert.show();
    }
}
