package twisk.vues;

import javafx.animation.PauseTransition;
import javafx.scene.control.Alert;
import javafx.scene.input.*;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.SnapshotParameters;
import javafx.scene.image.WritableImage;
import javafx.util.Duration;
import javafx.util.Pair;
import twisk.exceptions.ArcException;
import twisk.modele.MondeIG;
import twisk.modele.PointDeControleIG;

/**
 * Vue d'un point de contrôle
 */
public class VuePointDeControleIG extends Circle {
    private final PointDeControleIG pointDeControle;
    private final MondeIG monde;

    /**
     * Constructeur
     * @param pointDeControle le point de contrôle à représenter
     * @param monde le monde auquel appartient le point de contrôle
     */
    public VuePointDeControleIG(PointDeControleIG pointDeControle, MondeIG monde) {
        super(pointDeControle.getPosX(), pointDeControle.getPosY(), 5); // Rayon de 5 pixels
        this.pointDeControle = pointDeControle;
        this.monde = monde;

        // Style du point de contrôle
        this.setFill(Color.LIGHTBLUE);
        this.setStroke(Color.BLUE);
        this.setStrokeWidth(1);

        // Ajouter une classe CSS pour le styling
        this.getStyleClass().add("point-controle");

        // Configurer les interactions
        configurerInteractions();
    }

    /**
     * Configure les interactions du point de contrôle
     */
    private void configurerInteractions() {
        // Configurer le point comme source de drag
        this.setOnDragDetected(this::gererDebutDrag);

        // Configurer le point comme cible de drop
        this.setOnDragOver(this::gererDragOver);
        this.setOnDragDropped(this::gererDrop);

        // Ajouter un écouteur de clic
        this.setOnMouseClicked(event -> {
            try {
                monde.selectionnerPointDeControle(pointDeControle);
            } catch (ArcException e) {
                afficherMessageErreur(e.getMessage());
            }
            event.consume();
        });

    }

    /**
     * Gère le début du drag
     * @param event l'événement de début de drag
     */
    private void gererDebutDrag(MouseEvent event) {
        // Démarrer le drag
        Dragboard db = this.startDragAndDrop(TransferMode.ANY);

        // Créer un contenu pour le drag avec l'ID du point de contrôle
        ClipboardContent content = new ClipboardContent();
        content.putString(pointDeControle.getId());
        db.setContent(content);

        // Créer une image pour le drag (aspect visuel uniquement)
        SnapshotParameters params = new SnapshotParameters();
        params.setFill(Color.TRANSPARENT);
        WritableImage image = this.snapshot(params, null);
        db.setDragView(image);

        event.consume();
    }

    /**
     * Gère le survol du drag
     * @param event l'événement de survol
     */
    private void gererDragOver(DragEvent event) {
        // Vérifier que la source du drag est un autre point de contrôle
        if (event.getGestureSource() instanceof VuePointDeControleIG &&
                event.getGestureSource() != this) {
            // Accepter le transfert
            event.acceptTransferModes(TransferMode.LINK);
        }

        event.consume();
    }

    /**
     * Gère le drop
     * @param event l'événement de drop
     */
    private void gererDrop(DragEvent event) {
        // Récupérer l'ID du point de contrôle source
        String idPointSource = event.getDragboard().getString();

        // Récupérer le point de contrôle source à partir de son ID
        PointDeControleIG pointSource = monde.getPointDeControleById(idPointSource);

        if (pointSource != null) {
            // Déléguer la logique de création d'arc au modèle
            Pair<Boolean, String> resultat = monde.gererDragAndDropPoints(pointSource, this.pointDeControle);

            // Si l'opération a échoué, afficher un message d'erreur
            if (!resultat.getKey()) {
                afficherMessageErreur(resultat.getValue());
            }
        }

        // Indiquer que le drop est terminé
        event.setDropCompleted(true);
        event.consume();
    }

    /**
     * Affiche un message d'erreur (aspect UI uniquement)
     * @param message le message à afficher
     */
    private void afficherMessageErreur(String message) {
        // Créer une boîte de dialogue d'alerte
        Alert alert = new Alert(Alert.AlertType.ERROR);
        alert.setTitle("Erreur");
        alert.setHeaderText("Impossible de créer l'arc");
        alert.setContentText(message);

        // Configurer la fermeture automatique après 3 secondes
        PauseTransition delay = new PauseTransition(Duration.seconds(3));
        delay.setOnFinished(event -> alert.close());
        delay.play();

        // Afficher la boîte de dialogue
        alert.show();
    }
}
